import os
import peewee
import requests
import datetime
import xml.etree.cElementTree as et


WANTED = 0
FOUND = 1
DOWNLOADING = 2
DOWNLOADED = 3
SKIPPED = 4

VIDEO_EXTENSIONS = (
    'avi', 'mkv', 'mpg', 'mpeg', 'wmv', 'ogm', 'mp4',
    'iso', 'img', 'divx', 'm2ts', 'm4v', 'ts', 'flv',
    'f4v', 'mov', 'rmvb', 'vob', 'dvr-ms', 'wtv', 'ogv',
    '3gp'
)

dirname = os.path.dirname(__file__)
db = peewee.SqliteDatabase('snor_database.db')


class Show(peewee.Model):
    tvdb_id = peewee.IntegerField(unique=True)
    seriesname = peewee.CharField()
    status = peewee.CharField()
    folder = peewee.CharField()
    network = peewee.CharField(null=True)
    language = peewee.CharField(null=True)
    overview = peewee.TextField(null=True)
    filters = peewee.TextField(null=True)
    imdb_id = peewee.CharField(null=True)
    zap2it_id = peewee.CharField(null=True)
    date_added = peewee.DateTimeField(default=datetime.datetime.now)
    date_updated = peewee.DateTimeField(null=True)
    firstaired = peewee.DateTimeField(null=True)
    date_last_updated = peewee.DateTimeField(default=datetime.datetime.now)
    airs_time = peewee.CharField(null=True)
    airs_dayofweek = peewee.CharField(null=True)
    is_active = peewee.BooleanField(default=True)
    use_season_folders = peewee.BooleanField(default=False)

    def check_download_status(self):
        vids = []

        episodes = Episode.select().where(
            Episode.show == self,
            Episode.status != DOWNLOADED
        )

        if episodes.count() != 0:
            for root, dirs, files in os.walk(self.folder):
                files = [os.path.join(root, f) for f in files]
                # Only copy video files
                vids.extend(
                    filter(lambda x: x.lower().endswith(VIDEO_EXTENSIONS), files)
                )

        for e in episodes:
            l = lambda x: e.get_code() in os.path.basename(x).upper()
            found = filter(l, vids)
            if found:
                e.location = found[0]
                e.status = DOWNLOADED
                e.save()

    def save_episodes(self, skip=False):
        status = WANTED
        if skip:
            status = SKIPPED

        for episode in self._get_episodes():
            # Only create episode if it doesn't exist
            eps = Episode.select().where(Episode.tvdb_id == episode['tvdb_id'])
            if eps.count() == 0:
                Episode.create(
                    show=self,
                    status=status,
                    **episode
                )
            else:
                episode_id = list(eps)[0].id
                Episode.update(**episode)\
                    .where(Episode.id == episode_id).execute()

        #After saving mark unaired episodes as wanted
        if skip:
            Episode.update(status=WANTED).where(
                Episode.status == SKIPPED,
                Episode.show == self,
                Episode.firstaired > datetime.datetime.now()
            ).execute()

    def mark_completed(self):
        #some_path = '/Volumes/media/Series/Mad Men'
        files = []
        for directory, dirnames, filenames in walk(self.folder):
            files.append([f.lower() for f in filenames])
        return files

    def _get_episodes(self):
        result = []
        url = 'http://thetvdb.com/api/103048D30C58E1F3/series/{id}/all/'
        r = requests.get(url.format(id=self.tvdb_id))
        tree = et.fromstring(r.content)
        # Only use valid keys
        vk = Episode._meta.fields.keys()
        for ch in tree.findall('Episode'):
            e = {i.tag.lower(): i.text for i in ch if i.tag.lower() in vk}
            e['tvdb_id'] = e['id']
            del e['id']
            result.append(e)
        return result

    def __unicode__(self):
        return self.seriesname

    class Meta:
        database = db


class Episode(peewee.Model):
    tvdb_id = peewee.IntegerField(unique=True)
    imdb_id = peewee.CharField(null=True)
    episodename = peewee.CharField(null=True)
    location = peewee.CharField(null=True)
    seasonnumber = peewee.IntegerField()
    episodenumber = peewee.IntegerField()
    overview = peewee.TextField(null=True)
    firstaired = peewee.DateTimeField(null=True)
    date_added = peewee.DateTimeField(default=datetime.datetime.now)
    date_last_updated = peewee.DateTimeField(default=datetime.datetime.now)
    magnet_hash = peewee.CharField(null=True)
    status = peewee.IntegerField(default=0)
    show = peewee.ForeignKeyField(Show)

    def get_status(self):
        s = ['wanted', 'found', 'downloading', 'downloaded', 'skipped']
        if self.status not in [DOWNLOADED, DOWNLOADING]:
            if not self.firstaired or self.firstaired > datetime.datetime.now():
                return 'not aired'
        return s[self.status]

    def get_magnet_link(self):
        return 'magnet:?xt=urn:btih:{h}'.format(h=self.magnet_hash)

    def get_download_dir(self):
        directory = self.show.folder
        return directory

    def get_code(self):
        ep = self.episodenumber
        if ep < 10:
            ep = '0{e}'.format(e=ep)
        se = self.seasonnumber
        if se < 10:
            se = '0{s}'.format(s=se)
        return 'S{s}E{e}'.format(s=se, e=ep)

    def __unicode__(self):
        return '{s}: {e} {c}'.format(
            s=self.show.seriesname,
            e=self.episodename,
            c=self.get_code()
        )

    class Meta:
        database = db


class Setting(peewee.Model):
    name = peewee.CharField(null=True)
    value = peewee.TextField(null=True)
    date_added = peewee.DateTimeField(default=datetime.datetime.now)
    date_last_updated = peewee.DateTimeField(default=datetime.datetime.now)

    class Meta:
        database = db



import requests
import re
from . import BaseCLient


class Client(BaseCLient):
    _name = 'utorrent'
    _settings = {
        'port': 61137,
        'address': 'localhost',
        'user': 'admin',
        'password': 'password'
    }

    def add_magnet_hash(self, magnet_hash, download_dir):
        auth = (self._settings['user'], self._settings['password'])
        url = 'http://{address}:{port}/gui/'.format(**self._settings)
        r = requests.get(url + 'token.html', auth=auth)
        cookie = r.cookies
        # Extract token
        token_re = "<div id='token' style='display:none;'>([^<>]+)</div>"
        match = re.search(token_re, r.content)
        token = match.group(1)

        #Set to correct download folder
        p = {
            'token': token,
            'action': 'setsetting',
            's': 'dir_completed_download',
            'v': download_dir
        }
        r = requests.get(url, params=p, auth=auth, cookies=cookie)
        p = {
            'token': token,
            'action': 'setsetting',
            's': 'dir_completed_download_flag',
            'v': '1'
        }
        requests.get(url, params=p, auth=auth, cookies=cookie)
        # Add magnet hash
        magnet_url = 'magnet:?xt=urn:btih:{0}'.format(magnet_hash)
        p = {'token': token, 'action': 'add-url', 's': magnet_url}
        r = requests.get(url, params=p, auth=auth, cookies=cookie)

    def get_completed(self):
        print 'Completed'

    def is_active(self):
        return True



import os


class BaseCLient():
    name = 'base'
    _result_keys = ['size', 'seeds', 'peers']

    def __init__(self):
        """
        Dont initialize client because it could raise an error
        """
        pass

    @property
    def valid_filters(self):
        """
        Returns list with valid filter keys name and hash are mandatory.
        """
        filters = self._result_keys
        filters.extend(['name', 'hash'])
        return filters

    def search(self, show_name, episode_code):
        """
        Returns a list with dicts with the search results .

        Dict must contain the keys returned by `valid_filters`

        :param show_name: The name of the show to search for
        :param episode_code: The episode en season number (Ex. S01E12).
        """
        raise NotImplementedError

    def is_active(self):
        raise NotImplementedError


def get_search_client(client_name):
    module = 'snor.search.' + client_name
    try:
        m = __import__(module, globals(), locals(), ['Client'], -1)
        client = m.Client()
        return client
    except ImportError, ex:
        print str(ex)
        msg = 'Could not load search client {c}'.format(c=client_name)
        raise ImportError(msg)


def get_search_clients():
    files = []
    dirname = os.path.dirname(__file__)
    for f in os.listdir(dirname):
        if f.endswith('.py') and f != '__init__.py':
            files.append(f[:-3])
    return files



import os
import json
import datetime
import clients
import search
from models import Setting


class Conf():
    setting_name = 'snor.conf'
    _settings = {
        'login_required': False,
        'username': 'admin',
        'password': 'admin',
        'filters': '',
        'secret_key': os.urandom(24).encode('base64'),
        'folder': os.path.expanduser('~'),
        'client': clients.get_torrent_clients()[0],
        'search_client': search.get_search_clients()[0],
        'use_season_folders': True,
        'download_new_only': False
    }

    def __getattr__(self, name):
        settings = self.get_settings()
        return settings[name]

    def get_settings(self):
        try:
            setting = Setting.get(name=self.setting_name)
            return json.loads(setting.value)
        except:
            Setting.create(
                name=self.setting_name,
                value=json.dumps(self._settings)
            )
        return self._settings

    def set_settings(self, **kwargs):
        # Only copy valid keys fallback to default settings
        vk = self._settings.keys()
        kwargs = {k: v for k, v in kwargs.items() if k in vk}
        kwargs = dict(self._settings.items() + kwargs.items())
        try:
            s = Setting.get(name=self.setting_name)
            s.value = json.dumps(kwargs)
            s.save()
        except:
            Setting.create(
                name=self.setting_name,
                value=json.dumps(kwargs),
                date_last_updated=datetime.datetime.now()
            )

settings = Conf()



# -*- coding: utf-8 -*-
import os
import sys

try:
    from setuptools import setup
    setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

setup(
    name='snor',
    version='0.0.4',
    author='Paul Tax',
    author_email='paultax@gmail.com',
    include_package_data=True,
    install_requires = [
        'requests>=0.14.0',
        'transmissionrpc>=0.9',
        'peewee>=2.0.8',
        'Flask>=0.9',
        'Jinja2>=2.6'
    ],
    packages=[
        'snor',
        'snor.search',
        'snor.clients'
    ],
    entry_points={
        'console_scripts': [
            'snor = snor.server:main',
        ],
    },
    url='https://github.com/tax/snor',
    license='MIT licence, see LICENCE.txt',
    description='A very lightweight "sickbeard for torrents" ',
    long_description=open('README.md').read(),
)



import transmissionrpc
from . import BaseCLient


class Client(BaseCLient):
    _name = 'transmission'
    _settings = {
        'port': 9091,
        'address': 'localhost',
        'user': None,
        'password': None
    }

    def add_magnet_hash(self, magnet_hash, download_dir):
        c = transmissionrpc.Client(**self.get_settings())
        url = 'magnet:?xt=urn:btih:{h}'.format(h=magnet_hash)
        return c.add_torrent(url, download_dir=download_dir)

    def get_completed(self):
        print 'Completed'

    def is_active(self):
        try:
            transmissionrpc.Client(**self.get_settings())
        except:
            return False
        return True



import os
import json
import datetime
from ..models import Setting


class BaseCLient():
    _settings = {}
    _name = 'base'

    def __init__(self):
        # Dont initialize client because it could raise an error
        pass

    @property
    def setting_name(self):
        return 'client.{0}.conf'.format(self._name)

    def add_magnet_hash(self, hash, dowload_dir):
        raise NotImplementedError

    def get_completed(self):
        raise NotImplementedError

    def is_active(self):
        raise NotImplementedError

    def get_settings(self):
        try:
            setting = Setting.get(name=self.setting_name)
            return json.loads(setting.value)
        except:
            Setting.create(
                name=self.setting_name,
                value=json.dumps(self._settings)
            )
        return self._settings

    def set_settings(self, **kwargs):
        # Only copy valid keys fallback to default settings
        vk = self._settings.keys()
        kwargs = {k: v for k, v in kwargs.items() if k in vk}
        kwargs = dict(self._settings.items() + kwargs.items())
        try:
            s = Setting.get(name=self.setting_name)
            s.value = json.dumps(kwargs)
            s.save()
        except:
            Setting.create(
                name=self.setting_name,
                value=json.dumps(kwargs),
                date_last_updated=datetime.datetime.now()
            )


def get_torrent_client(client_name):
    module = 'snor.clients.' + client_name
    try:
        m = __import__(module, globals(), locals(), ['Client'], -1)
        client = m.Client()
        return client
    except ImportError, ex:
        msg = 'Could not load torrent client {c}'.format(c=client_name)
        raise ImportError(msg)


def get_torrent_clients():
    files = []
    dirname = os.path.dirname(__file__)
    for f in os.listdir(dirname):
        if f.endswith('.py') and f != '__init__.py':
            files.append(f[:-3])
    return files



import requests
import xml.etree.cElementTree as et
from . import BaseCLient


class Client(BaseCLient):
    name = 'torrentz.eu'

    def search(self, show_name, episode_code):
        result = []
        url = 'http://torrentz.eu/feed?q={0}+{1}'

        r = requests.get(url.format(show_name, episode_code))
        t = et.fromstring(r.content)

        for ch in t.findall('channel/item'):
            d = {item.tag.lower(): item.text for item in ch}

            res = {'name': d['title']}
            split = d['description'].split(' ')
            if len(split) == 9:
                res['size'] = split[1]
                res['hash'] = split[8]
                res['seeds'] = split[4]
                res['peers'] = split[6]

                result.append(res)

        return result



# -*- coding: utf-8 -*-
import os
import urllib
import logging
import argparse
from logging.handlers import RotatingFileHandler
from functools import wraps
from flask import Flask
from flask import jsonify, redirect, request, render_template, session, make_response
from conf import settings
from models import Episode, Show
import utils
import models
import clients
import search


LOGFILE_NAME = 'debug.log'
MB = 1024 * 1024 * 1024

app = Flask(__name__)
app.secret_key = 'SECRET_FOR_TESTING'
app.config['LOGGER_NAME'] = 'snor_log'

# Create logger
handler = RotatingFileHandler(LOGFILE_NAME, maxBytes=MB, backupCount=1)
frmt = logging.Formatter("%(asctime)s %(levelname)s\t%(message)s")
handler.setFormatter(frmt)
handler.setLevel(logging.INFO)
app.logger.addHandler(handler)


def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if settings.login_required and 'username' not in session:
            return redirect('/login/')
        return f(*args, **kwargs)
    return decorated_function


@app.route('/login/', methods=['GET', 'POST'])
def login():
    msg = request.args.get('msg', '')
    if request.method == 'POST':
        if not settings.login_required:
            return redirect('/')
        form = request.form.to_dict()
        u = settings.username
        p = settings.password
        if form['username'] == u and form['password'] == p:
            session['username'] = u
            return redirect('/')
        msg = 'Invalid username or password'
    return render_template('login.html', msg=msg)


@app.route('/logout/')
def logout():
    if 'username' in session:
        del session['username']
    return redirect('/login/?msg=Logged out')


@app.route('/')
@login_required
def index():
    return render_template('home.html', shows=list(Show.select()))


@app.route('/log/')
@login_required
def logfile():
    response = make_response(open(LOGFILE_NAME).read())
    response.headers["Content-type"] = "text/plain"
    return response


@app.route('/settings/', methods=['GET', 'POST'])
@login_required
def configuration():
    c = {
        'clients': clients.get_torrent_clients(),
        'search_clients': search.get_search_clients(),
        'settings': settings
    }

    if request.method == 'POST':
        try:
            new = request.form.to_dict()
            new['login_required'] = new['login_required'] == 'login'
            new['use_season_folders'] = 'use_season_folders' in new
            new['download_new_only'] = 'download_new_only' in new
            # Dont overwrite secret key
            new['secret_key'] = app.secret_key
            settings.set_settings(**new)
            c['msg'] = 'Saved settings'
        except Exception, ex:
            c['msg'] = str(ex)
    return render_template('settings.html', **c)


@app.route('/show/add/')
@login_required
def show_add_choice():
    return render_template('show_add_choice.html')


@app.route('/show/<int:show_id>/')
@login_required
def get_show(show_id):
    c = {
        'show': Show.get(id=show_id),
        'episodes': Episode.select().join(Show).where(Show.id == show_id)
    }
    return render_template('show.html', **c)


@app.route('/show/<int:show_id>/settings/', methods=['GET', 'POST'])
@login_required
def get_show_settings(show_id):
    s = Show.get(id=show_id)
    c = {
        'show': s,
        'settings': settings,
        'choice': 'saved_show'
    }
    if request.method == 'POST':
        s.folder = request.form['folder']
        s.filters = request.form['filters']
        s.use_season_folders = 'use_season_folders' in request.form
        s.save()
        msg = 'Successfully saved settings'
        return redirect('/show/{id}/?msg={msg}'.format(id=show_id, msg=msg))
    return render_template('show_add.html', **c)


@app.route('/show/add/<choice>/', methods=['GET', 'POST'])
@login_required
def search_show(choice):
    if request.method == 'POST' or 'folder' in request.args:
        if 'folder' in request.args:
            q = os.path.basename(request.args.get('folder', ''))
        else:
            q = request.form['q']
        shows = utils.find_shows(q)
        return render_template(
            'search_result.html',
            result=shows,
            choice=choice,
            query=q
        )
    return render_template('search.html', choice=choice)


@app.route('/show/add/<choice>/<int:show_id>/', methods=['GET', 'POST'])
@login_required
def add_show(choice, show_id):
    if request.method == 'POST':
        pk = utils.save_show(choice=choice, **request.form.to_dict())
        return redirect('/show/{id}/'.format(id=pk))
    kwargs = {
        'choice': choice,
        'show_id': show_id,
        'settings': settings
    }
    return render_template('show_add.html', **kwargs)


@app.route('/list/dir/')
@login_required
def list_dir():
    res = utils.list_directory(request.args.get('folder', settings.folder))
    return jsonify(**res)


@app.route('/api/', methods=['POST', 'GET'])
@login_required
def api(**kwargs):
    args = []
    action = request.args.get('action', '')

    def scan_show(show):
        return show.check_download_status()

    def delete_show(show):
        show.delete_instance(recursive=True)

    def update_show(show):
        return utils.save_show(tvdb_id=show.tvdb_id)

    def episode_mark_skipped(episode):
        episode.status = models.SKIPPED
        episode.save()

    def episode_mark_wanted(episode):
        episode.status = models.WANTED
        episode.save()

    def episode_mark_downloaded(episode, location):
        episode.status = models.DOWNLOADED
        episode.location = location
        episode.save()

    actions = {
        'scan': scan_show,
        'delete': delete_show,
        'update': update_show,
        'episode_mark_wanted': episode_mark_wanted,
        'episode_mark_skipped': episode_mark_skipped,
        'episode_mark_downloaded': episode_mark_downloaded,
        'background_search': utils.process_search_torrent,
        'background_download': utils.process_download_torrent,
        'background_update': utils.process_check_new_episodes,
        'background_status': utils.process_check_downloaded
    }

    if not action in actions:
        msg = 'No action named {}'.format(action)
        return jsonify(stat='fail', msg=msg, result=[])

    if 'show_id' in request.args:
        args.append(Show.get(id=request.args['show_id']))
    if 'episode_id' in request.args:
        args.append(Episode.get(id=request.args['episode_id']))
    if 'location' in request.args:
        args.append(request.args['location'])

    res = actions[action](*args)
    return jsonify(stat='ok', result=res)


@app.template_filter('u')
@app.template_filter('urlencode')
def urlencode_filter(s):
    s = s.encode('utf8')
    s = urllib.quote_plus(s)
    return s


def main():
    # Create db and tables if it doesn't exist
    parser = argparse.ArgumentParser()
    parser.add_argument('--port', default=5555, type=int)
    parser.add_argument('--host', default='0.0.0.0')
    parser.add_argument('--debug', action='store_true')
    args = parser.parse_args()

    utils.create_database()
    utils.start_background_tasks(args.host, args.port)
    # Start webserver
    app.secret_key = str(settings.secret_key)
    app.run(
        host=args.host,
        port=args.port,
        debug=args.debug,
        use_reloader=False
    )



import os
import sys
import signal
import time
import ctypes
import itertools
import string
import platform
import threading
import json
import datetime
import xml.etree.cElementTree as et
import requests
import search
import clients
import logging
import models
from models import Episode, Show, Setting
from conf import settings


logger = logging.getLogger('snor_log')
API_KEY = '103048D30C58E1F3'
IS_WINDOWS = 'Windows' in platform.system()
DB_NAME = 'snor_database.db'


def find_shows(q):
    url = 'http://thetvdb.com/api/GetSeries.php?seriesname={q}'
    logger.info(url.format(q=q))
    r = requests.get(url.format(q=q))
    t = et.fromstring(r.content)
    return [{i.tag.lower(): i.text for i in c} for c in t.findall('Series')]


def save_show(choice='new', **kwargs):
    # Convert to boolean
    kwargs['use_season_folders'] = 'use_season_folders' in kwargs

    # Download serie info from tvdb
    url = 'http://thetvdb.com/api/{api_key}/series/{id}/'

    r = requests.get(url.format(id=kwargs['tvdb_id'], api_key=API_KEY))
    t = et.fromstring(r.content)
    show = {i.tag.lower(): i.text for i in t.find('Series')}

    # Only create show in db if it doesn't exist
    shows = Show.select().where(Show.tvdb_id == kwargs['tvdb_id'])
    if shows.count() == 0:
        s = Show.create(**dict(kwargs.items() + show.items()))
    else:
        s = list(shows)[0]
        # Only use valid keys
        vk = Show._meta.fields.keys()
        d = {k: v for k, v in dict(kwargs.items() + show.items()).items() if k in vk}
        d['tvdb_id'] = d['id']
        del d['id']
        d['date_last_updated'] = datetime.datetime.now()
        s.update(**d).execute()

    s.save_episodes('download_new_only' in kwargs)

    # Mark episodes as downloaded if they exist
    if choice == 'existing':
        s.check_download_status()

    return s.id


def process_search_torrent():
    res = []
    eps = Episode.select().join(Show).where(
        Show.is_active == True,
        Episode.status == models.WANTED,
        Episode.firstaired < datetime.datetime.now()
    )

    for e in eps:
        c = search.get_search_client(settings.search_client)
        torrents = c.search(e.show.seriesname, e.get_code())

        # Create default filters for episode code
        fns = [{'key': 'name', 'value': e.get_code(), 'operator': 'in'}]
        # Add filters from show settings
        fns.extend(json.loads(e.show.filters))
        filters = [build_filter(**f) for f in fns
                   if f['key'] in c.valid_filters]
        for fl in filters:
            torrents = filter(fl, torrents)

        msg = {'msg': 'No hash found for {e}'.format(e=e), 'stat': 'fail'}
        if len(torrents) > 0:
            e.magnet_hash = torrents[0]['hash']
            e.status = models.FOUND
            e.save()
            msg['msg'] = 'Saved hash for {e}'.format(e=e)
            msg['stat'] = 'ok'
        res.append(msg)

    return res


def process_download_torrent():
    res = []
    eps = Episode.select().join(Show).where(
        Show.is_active == True,
        Episode.status == models.FOUND
    )
    tc = clients.get_torrent_client(settings.client)
    for e in eps:
        try:
            tc.add_magnet_hash(e.magnet_hash, e.get_download_dir())
            e.status = models.DOWNLOADING
            e.save()
            res.append({
                'msg': 'Started downloading {e}'.format(e=e),
                'stat': 'ok'
            })
        except Exception, ex:
            res.append({
                'msg': 'Failed to add {e} to client: {ex}'.format(e=e, ex=ex),
                'stat': 'fail'
            })
    return res


def process_check_downloaded():
    shows = Show.select().join(Episode).where(
        Episode.status == models.WANTED,
        Episode.firstaired < datetime.datetime.now()
    ).group_by(Show)
    return [s.check_download_status() for s in shows]


def process_check_new_episodes():
    res = []
    # Only check after 24 hours
    yesterday = datetime.datetime.now() - datetime.timedelta(1)
    shows = Show.select().where(
        Show.date_last_updated > yesterday
    )
    for s in shows:
        res.append(save_show(tvdb_id=s.tvdb_id))
    return res


def build_filter(key, value, operator):

    if operator == 'eq':
        return lambda x: x[key] == value
    if operator == 'ne':
        return lambda x: x[key] != value
    if operator == 'gt':
        return lambda x: x[key] > value
    if operator == 'st':
        return lambda x: x[key] < value
    if operator == 'in':
        return lambda x: value.lower() in x[key].lower()
    if operator == 'not_in':
        return lambda x: value.lower() not in x[key].lower()
    if operator == 'startswith':
        return lambda x: value.lower().startswith(x[key].lower())
    if operator == 'not_startswith':
        return lambda x: not value.lower().startswith(x[key].lower())


def list_directory(current):
    res = {
        'result': [],
        'current': current,
        'selectable': True,
        'stat': 'ok'
    }

    if IS_WINDOWS and current == '/':
        for d in get_available_drives():
            f = {'path': d + ':\\', 'name': d + ':\\', 'is_dir': True}
            res['result'].append(f)
            res['selectable'] = False
        return res

    path = os.path.abspath(os.path.join(current, os.pardir))
    if len(current) == 3 and current.endswith(':\\'):
        path = '/'
        f = {'path': path, 'name': '..', 'is_dir': True}
        res['result'].append(f)
    elif current != '/':
        f = {'path': path, 'name': '..', 'is_dir': True}
        res['result'].append(f)

    try:
        for item in os.listdir(current):
            f = {
                'path': os.path.join(current, item),
                'name': item,
                'is_dir': os.path.isdir(os.path.join(current, item))
            }
            res['result'].append(f)
    except:
        res['selectable'] = False
        res['msg'] = 'Could not open this directory'
        res['stat'] = 'fail'
    return res


def get_available_drives():
    if not IS_WINDOWS:
        return []
    drive_bitmask = ctypes.cdll.kernel32.GetLogicalDrives()
    return list(
        itertools.compress(
            string.ascii_uppercase,
            map(lambda x: ord(x) - ord('0'), bin(drive_bitmask)[:1:-1])
        )
    )


def create_database(destroy_existing=False):
    """ Create db and tables if it doesn't exist """
    if not os.path.exists(DB_NAME):
        logger.info('Create database: {0}'.format(DB_NAME))
        open(DB_NAME, 'a').close()
        Show.create_table()
        Episode.create_table()
        Setting.create_table()


class Tasks(threading.Thread):
    def __init__(self, tasks, seconds, host, port):
        threading.Thread.__init__(self)
        self.quit = False
        self.tasks = tasks
        self.seconds = seconds
        self.host = host
        self.port = port

    def run(self):
        counter = 0
        while not self.quit:
            if counter == self.seconds:
                counter = 0
                logger.info('Run background tasks')
                for t in self.tasks:
                    url = 'http://{host}:{port}/api/?action={task}'
                    url = url.format(task=t, host=self.host, port=self.port)
                    r = requests.post(url)
                    logger.info(r.content)
            counter += 1
            time.sleep(1)
        logger.info('Quit running background tasks')

    def stop(self):
        self.quit = True


def start_background_tasks(host, port):
    # Start background tasks to search and download
    tasks = [
        'background_search', 'background_download',
        'background_status', 'background_update'
    ]
    t = Tasks(tasks, host=host, port=port, seconds=5 * 30)
    t.start()

    def signal_exit(signal, frame):
        t.stop()
        sys.exit(0)

    signal.signal(signal.SIGINT, signal_exit)
